﻿using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Logging.Console;
using System;

namespace NetConfIT2018.EF.Data
{
	public partial class northwindContext : DbContext
	{
		public static readonly LoggerFactory MyLoggerFactory = new LoggerFactory(new[] { new ConsoleLoggerProvider((category, level)
						=> category == DbLoggerCategory.Database.Command.Name
							 && level == LogLevel.Information, true) });

		public northwindContext()
		{
		}

		public northwindContext(DbContextOptions<northwindContext> options)
				: base(options)
		{
		}

		public virtual DbSet<Customers> Customers { get; set; }
		public virtual DbSet<OrderDetails> OrderDetails { get; set; }
		public virtual DbSet<Orders> Orders { get; set; }
		public virtual DbQuery<ProductSalesFor1997> ProductSalesFor1997 { get; set; }

		protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
		{
			if (!optionsBuilder.IsConfigured)
			{
				optionsBuilder.UseSqlServer("Server=(local);Database=northwind;Trusted_Connection=True;");
				optionsBuilder.UseLoggerFactory(MyLoggerFactory);
				//optionsBuilder.UseLazyLoadingProxies();
			}
		}

		protected override void OnModelCreating(ModelBuilder modelBuilder)
		{
			modelBuilder.Query<ProductSalesFor1997>().ToView("Product Sales for 1997");

			modelBuilder.Entity<Customers>(entity =>
			{
				entity.HasKey(e => e.CustomerId);

				entity.HasIndex(e => e.City)
									.HasName("City");

				entity.HasIndex(e => e.CompanyName)
									.HasName("CompanyName");

				entity.HasIndex(e => e.PostalCode)
									.HasName("PostalCode");

				entity.HasIndex(e => e.Region)
									.HasName("Region");

				entity.Property(e => e.CustomerId)
									.HasColumnName("CustomerID")
									.HasMaxLength(5)
									.ValueGeneratedNever();

				entity.Property(e => e.Address).HasMaxLength(60);

				entity.Property(e => e.City).HasMaxLength(15);

				entity.Property(e => e.CompanyName)
									.IsRequired()
									.HasMaxLength(40);

				entity.Property(e => e.ContactName).HasMaxLength(30);

				entity.Property(e => e.ContactTitle).HasMaxLength(30);

				entity.Property(e => e.Country).HasMaxLength(15);

				entity.Property(e => e.Fax).HasMaxLength(24);

				entity.Property(e => e.Phone).HasMaxLength(24);

				entity.Property(e => e.PostalCode).HasMaxLength(10);

				entity.Property(e => e.Region).HasMaxLength(15);
			});

			modelBuilder.Entity<OrderDetails>(entity =>
			{
				entity.HasKey(e => new { e.OrderId, e.ProductId });

				entity.ToTable("Order Details");

				entity.HasIndex(e => e.OrderId)
									.HasName("OrdersOrder_Details");

				entity.HasIndex(e => e.ProductId)
									.HasName("ProductsOrder_Details");

				entity.Property(e => e.OrderId).HasColumnName("OrderID");

				entity.Property(e => e.ProductId).HasColumnName("ProductID");

				entity.Property(e => e.Quantity).HasDefaultValueSql("((1))");

				entity.Property(e => e.UnitPrice).HasColumnType("money");

				entity.HasOne(d => d.Order)
									.WithMany(p => p.OrderDetails)
									.HasForeignKey(d => d.OrderId)
									.OnDelete(DeleteBehavior.ClientSetNull)
									.HasConstraintName("FK_Order_Details_Orders");
			});

			modelBuilder.Entity<Orders>(entity =>
			{
				entity.HasKey(e => e.OrderId);

				entity.HasIndex(e => e.CustomerId)
									.HasName("CustomersOrders");

				entity.HasIndex(e => e.EmployeeId)
									.HasName("EmployeesOrders");

				entity.HasIndex(e => e.OrderDate)
									.HasName("OrderDate");

				entity.HasIndex(e => e.ShipPostalCode)
									.HasName("ShipPostalCode");

				entity.HasIndex(e => e.ShipVia)
									.HasName("ShippersOrders");

				entity.HasIndex(e => e.ShippedDate)
									.HasName("ShippedDate");

				entity.Property(e => e.OrderId).HasColumnName("OrderID");

				entity.Property(e => e.CustomerId)
									.HasColumnName("CustomerID")
									.HasMaxLength(5);

				entity.Property(e => e.EmployeeId).HasColumnName("EmployeeID");

				entity.Property(e => e.Freight)
									.HasColumnType("money")
									.HasDefaultValueSql("((0))");

				entity.Property(e => e.OrderDate).HasColumnType("datetime");

				entity.Property(e => e.RequiredDate).HasColumnType("datetime");

				entity.Property(e => e.ShipAddress).HasMaxLength(60);

				entity.Property(e => e.ShipCity).HasMaxLength(15);

				entity.Property(e => e.ShipCountry).HasMaxLength(15);

				entity.Property(e => e.ShipName).HasMaxLength(40);

				entity.Property(e => e.ShipPostalCode).HasMaxLength(10);

				entity.Property(e => e.ShipRegion).HasMaxLength(15);

				entity.Property(e => e.ShippedDate).HasColumnType("datetime");

				entity.HasOne(d => d.Customer)
									.WithMany(p => p.Orders)
									.HasForeignKey(d => d.CustomerId)
									.HasConstraintName("FK_Orders_Customers");

				//entity.Property(e => e.ShipViaString)
				//	.HasConversion(new EnumToStringConverter<ShipVia>());

				entity.Property(e => e.ShipViaString)
					.HasConversion(
						v => v.ToString(),
						v => (ShipVia)Enum.Parse(typeof(ShipVia), v));
			});
		}
	}
}
