﻿using NetConfIT2018.EF.Data;
using Newtonsoft.Json;
using System;
using System.Linq;
using System.Transactions;

namespace NetConfIT2018.EF
{
	class Program
	{

		static void Main(string[] args)
		{
			//Transaction();
			//QueryTypes();
			//Converter();
			//GroupBy();
			//LazyLoadingWithProxies();
			//LazyLoadingWithILazyLoader();
			//LazyLoadingWithAction();
			//QueryOptimization();
			Console.ReadLine();
		}

		private static void LazyLoadingWithProxies()
		{
			using (var ctx = new northwindContext())
			{
				var order = ctx.Orders.First();
				foreach (var item in order.OrderDetails)
				{
					Console.WriteLine(item.ProductId);
				}
			}
		}

		private static void LazyLoadingWithILazyLoader()
		{
			using (var ctx = new northwindContext())
			{
				var order = ctx.Orders.First();
				Console.WriteLine(order.OrderId);
				foreach (var item in order.OrderDetails)
				{
					Console.WriteLine(item.ProductId);
				}
			}
		}

		private static void LazyLoadingWithAction()
		{
			using (var ctx = new northwindContext())
			{
				var order = ctx.Orders.First();
				Console.WriteLine(order.OrderId);
				foreach (var item in order.OrderDetails)
				{
					Console.WriteLine(item.ProductId);
				}
			}
		}

		private static void GroupBy()
		{
			using (var ctx = new northwindContext())
			{
				var query = ctx.OrderDetails
					.GroupBy(o => new { o.Order.CustomerId })
					.Select(g => new
					{
						g.Key.CustomerId,
						Sum = g.Sum(d => d.Quantity),
						Count = g.Count()
					}).ToList();
				Console.WriteLine(JsonConvert.SerializeObject(query, Formatting.Indented));
			}
		}

		private static void Converter()
		{
			using (var ctx = new northwindContext())
			{
				var query = ctx.Orders.ToList();
				Console.WriteLine(query.Count);
			}
		}

		private static void Transaction()
		{
			using (var o = new TransactionScope())
			{
				using (var ctx = new northwindContext())
				{
					ctx.Customers.First(c => c.CustomerId == "ALFKI").Fax = DateTime.Now.Millisecond.ToString();
					ctx.SaveChanges();
				}

				using (var ctx = new northwindContext())
				{
					ctx.Customers.First(c => c.CustomerId == "ALFKI").Phone = DateTime.Now.Millisecond.ToString();
					ctx.SaveChanges();
				}
				o.Complete();
			}
		}

		static void QueryOptimization()
		{
			using (var ctx = new northwindContext())
			{
				var x = ctx.Customers
					.Select(c =>
					new
					{
						c.CustomerId,
						Orders = c.Orders.Select(o => o.OrderId).ToList()
					})
					.ToList();
				foreach (var item in x)
				{
					Console.WriteLine($"{item.CustomerId}-{String.Join(':', item.Orders)}");
				}
			}
		}
	}
}
